import logging
from dataclasses import asdict, dataclass
from typing import Dict, List, Optional

from databricks.sdk.core import Config, HeaderFactory, oauth_service_principal
from databricks.sql.client import Connection, connect

logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(name)s - %(message)s")


@dataclass
class DatabricksConnectionConfig:
    host: str
    http_path: str
    client_id: str
    client_secret: str
    catalog: str
    schema: str

    def as_dict(self) -> Dict[str, str]:
        return asdict(self)


class DatabricksQueryExecutor:
    def __init__(self, config: DatabricksConnectionConfig) -> None:
        self.config = config
        self.conn = self.__initialize()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.conn.close()

    def __initialize(self) -> Connection:
        conn = connect(
            server_hostname=self.config.host,
            http_path=self.config.http_path,
            credentials_provider=self.credential_provider,
        )
        return conn

    def get_query_results(self, query: str) -> List[Dict[str, str]]:
        results_asdict: List[Dict[str, str]] = []
        with self.conn.cursor() as cur:
            try:
                cur.execute(query)
                result: List[Dict[str, str]] = cur.fetchall()
            except Exception as e:
                logging.error(query, exc_info=True)
                logging.error("databricks get_query_results failed. %s", e)
                raise

            for row in result:
                results_asdict.append(row.asDict())
        return results_asdict

    def credential_provider(self) -> Optional[HeaderFactory]:
        config = Config(
            host=f"https://{self.config.host}", client_id=self.config.client_id, client_secret=self.config.client_secret
        )
        return oauth_service_principal(config)



"""
https://gist.github.com/orls/51525c86ee77a56ad396
Provides a utility to inject environment variables into argparse definitions.
Currently requires explicit naming of env vars to check for
"""

import argparse
import os


# Courtesy of http://stackoverflow.com/a/10551190 with env-var retrieval fixed
class EnvDefault(argparse.Action):
    """An argparse action class that auto-sets missing default values from env
    vars. Defaults to requiring the argument."""

    def __init__(self, envvar, required=True, default=None, **kwargs):
        # override values if envvar exists
        if envvar in os.environ:
            if kwargs.get("nargs", None) is None:
                default = os.environ[envvar]
            else:
                default = os.environ[envvar].split(" ")
        if required and default:
            required = False
        super(EnvDefault, self).__init__(default=default, required=required, **kwargs)

    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, values)


# functional sugar for the above
def env_default(envvar):
    def wrapper(**kwargs):
        return EnvDefault(envvar, **kwargs)

    return wrapper



import logging
from typing import Dict, List

from quollio_core.profilers.lineage import (
    gen_column_lineage_payload,
    gen_table_lineage_payload,
    parse_databricks_table_lineage,
)
from quollio_core.profilers.stats import gen_table_stats_payload
from quollio_core.repository import databricks, qdc

logger = logging.getLogger(__name__)


def databricks_table_level_lineage(
    conn: databricks.DatabricksConnectionConfig,
    qdc_client: qdc.QDCExternalAPIClient,
    tenant_id: str,
    dbt_table_name: str = "quollio_lineage_table_level",
) -> None:
    logging.basicConfig(level=logging.info, format="%(asctime)s - %(levelname)s - %(name)s - %(message)s")
    with databricks.DatabricksQueryExecutor(config=conn) as databricks_executor:
        results = databricks_executor.get_query_results(
            query=f"""
            SELECT
                DOWNSTREAM_TABLE_NAME,
                UPSTREAM_TABLES
            FROM {conn.catalog}.{conn.schema}.{dbt_table_name}
            """
        )
        tables = parse_databricks_table_lineage(results)
        update_table_lineage_inputs = gen_table_lineage_payload(
            tenant_id=tenant_id,
            endpoint=conn.host,
            tables=tables,
        )

        req_count = 0
        for update_table_lineage_input in update_table_lineage_inputs:
            logger.info(
                "Generating table lineage. downstream: %s -> %s-> %s",
                update_table_lineage_input.downstream_database_name,
                update_table_lineage_input.downstream_schema_name,
                update_table_lineage_input.downstream_table_name,
            )
            status_code = qdc_client.update_lineage_by_id(
                global_id=update_table_lineage_input.downstream_global_id,
                payload=update_table_lineage_input.upstreams.as_dict(),
            )
            if status_code == 200:
                req_count += 1
        logger.info("Generating table lineage is finished. %s lineages are ingested.", req_count)
    return


def databricks_column_level_lineage(
    conn: databricks.DatabricksConnectionConfig,
    qdc_client: qdc.QDCExternalAPIClient,
    tenant_id: str,
    dbt_table_name: str = "quollio_lineage_column_level",
) -> None:
    logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(name)s - %(message)s")
    with databricks.DatabricksQueryExecutor(config=conn) as databricks_executor:
        results = databricks_executor.get_query_results(
            query=f"""
            SELECT
                *
            FROM
                {conn.catalog}.{conn.schema}.{dbt_table_name}
            """
        )

    update_column_lineage_inputs = gen_column_lineage_payload(
        tenant_id=tenant_id,
        endpoint=conn.host,
        columns=results,
    )

    req_count = 0
    for update_column_lineage_input in update_column_lineage_inputs:
        logger.info(
            "Generating column lineage. downstream: %s -> %s -> %s -> %s",
            update_column_lineage_input.downstream_database_name,
            update_column_lineage_input.downstream_schema_name,
            update_column_lineage_input.downstream_table_name,
            update_column_lineage_input.downstream_column_name,
        )
        status_code = qdc_client.update_lineage_by_id(
            global_id=update_column_lineage_input.downstream_global_id,
            payload=update_column_lineage_input.upstreams.as_dict(),
        )
        if status_code == 200:
            req_count += 1
    logger.info(
        "Generating column lineage is finished. %s lineages are ingested.",
        req_count,
    )
    return


def _get_monitoring_tables(
    conn: databricks.DatabricksConnectionConfig, monitoring_table_id: str = "_profile_metrics"
) -> List[Dict[str, str]]:
    tables = []
    query = f"""
        SELECT
            table_catalog,
            table_schema,
            table_name,
            CONCAT(table_catalog, '.', table_schema, '.', table_name) AS table_fqdn
        FROM
            system.information_schema.tables
        WHERE table_name LIKE "%{monitoring_table_id}"
        """
    with databricks.DatabricksQueryExecutor(config=conn) as databricks_executor:
        tables = databricks_executor.get_query_results(query)
    if len(tables) > 0:
        logger.info("Found %s monitoring tables.", len(tables))
        return tables
    else:
        logger.info("No monitoring tables found.")
        return []


def _get_column_stats(
    conn: databricks.DatabricksConnectionConfig, monitoring_table_id: str = "_profile_metrics"
) -> List[Dict[str, str]]:
    tables = _get_monitoring_tables(conn, monitoring_table_id)
    if not tables:
        return []
    stats = []
    for table in tables:
        monitored_table = table["table_fqdn"].removesuffix("_profile_metrics")
        monitored_table = monitored_table.split(".")
        if len(monitored_table) != 3:
            raise ValueError(f"Invalid table name: {table['table_fqdn']}")
        with databricks.DatabricksQueryExecutor(config=conn) as databricks_executor:
            query = """
                WITH MaxCounts AS (
                    SELECT
                        t.COLUMN_NAME,
                        MAX(item.count) AS max_count,
                        MAX(t.window) AS latest
                    FROM
                        {monitoring_table} t
                    LATERAL VIEW EXPLODE(t.frequent_items) AS item
                    GROUP BY t.COLUMN_NAME
                )
                SELECT
                    "{monitored_table_catalog}" as DB_NAME,
                    "{monitored_table_schema}" as SCHEMA_NAME,
                    "{monitored_table_name}" as TABLE_NAME,
                    t.COLUMN_NAME,
                    t.DATA_TYPE,
                    t.distinct_count as CARDINALITY,
                    t.MAX as MAX_VALUE,
                    t.MIN as MIN_VALUE,
                    t.AVG as AVG_VALUE,
                    t.MEDIAN as MEDIAN_VALUE,
                    t.STDDEV as STDDEV_VALUE,
                    t.NUM_NULLS as NULL_COUNT,
                    item.item AS MODE_VALUE
                FROM
                    {monitoring_table} t
                JOIN MaxCounts mc ON t.COLUMN_NAME = mc.COLUMN_NAME
                LATERAL VIEW EXPLODE(t.frequent_items) AS item
                WHERE
                    item.count = mc.max_count
                    AND t.window = mc.latest
                """.format(
                monitoring_table=table["table_fqdn"],
                monitored_table_catalog=monitored_table[0],
                monitored_table_schema=monitored_table[1],
                monitored_table_name=monitored_table[2],
            )
            stats.append(databricks_executor.get_query_results(query))
    return stats


def databricks_column_stats(
    conn: databricks.DatabricksConnectionConfig,
    qdc_client: qdc.QDCExternalAPIClient,
    tenant_id: str,
    monitoring_table_id: str = "_profile_metrics",
) -> None:
    table_stats = _get_column_stats(conn, monitoring_table_id)
    for table in table_stats:
        stats = gen_table_stats_payload(tenant_id, conn.host, table)
        for stat in stats:
            status_code = qdc_client.update_stats_by_id(
                global_id=stat.global_id,
                payload=stat.body.as_dict(),
            )
            if status_code == 200:
                logger.info("Stats for %s is successfully ingested.", stat.global_id)
    return



import argparse
import logging
import os

from quollio_core.helper.core import setup_dbt_profile
from quollio_core.helper.env_default import env_default
from quollio_core.profilers.databricks import (
    databricks_column_level_lineage,
    databricks_column_stats,
    databricks_table_level_lineage,
)
from quollio_core.repository import databricks as db
from quollio_core.repository import dbt, qdc

logger = logging.getLogger(__name__)


def build_view(
    conn: db.DatabricksConnectionConfig,
    target_tables: str,
    log_level: str = "info",
) -> None:
    logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(name)s - %(message)s")

    logger.info("Build profiler views using dbt")
    # set parameters
    dbt_client = dbt.DBTClient()
    current_dir = os.path.dirname(os.path.abspath(__file__))
    project_path = f"{current_dir}/dbt_projects/databricks"
    template_path = f"{current_dir}/dbt_projects/databricks/profiles"
    template_name = "profiles_template.yml"

    # build views using dbt
    setup_dbt_profile(connections_json=conn.as_dict(), template_path=template_path, template_name=template_name)
    # FIXME: when executing some of the commands, directory changes due to the library bug.
    # https://github.com/dbt-labs/dbt-core/issues/8997
    dbt_client.invoke(
        cmd="deps",
        project_dir=project_path,
        profile_dir=template_path,
        options=["--no-use-colors", "--log-level", log_level],
    )

    run_options = ["--no-use-colors", "--log-level", log_level, "--select", target_tables]
    dbt_client.invoke(
        cmd="run",
        project_dir=project_path,
        profile_dir=template_path,
        options=run_options,
    )
    return


def load_lineage(
    conn: db.DatabricksConnectionConfig,
    qdc_client: qdc.QDCExternalAPIClient,
    tenant_id: str,
) -> None:
    logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(name)s - %(message)s")

    logger.info("Generate Databricks table to table lineage.")
    databricks_table_level_lineage(
        conn=conn, qdc_client=qdc_client, tenant_id=tenant_id, dbt_table_name="quollio_lineage_table_level"
    )

    logger.info("Generate Databricks column to column lineage.")
    databricks_column_level_lineage(
        conn=conn, qdc_client=qdc_client, tenant_id=tenant_id, dbt_table_name="quollio_lineage_column_level"
    )

    logger.info("Lineage data is successfully loaded.")
    return


def load_column_stats(
    conn: db.DatabricksConnectionConfig,
    qdc_client: qdc.QDCExternalAPIClient,
    tenant_id: str,
) -> None:
    logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(name)s - %(message)s")

    logger.info("Generate Databricks column stats.")
    databricks_column_stats(
        conn=conn,
        qdc_client=qdc_client,
        tenant_id=tenant_id,
    )

    logger.info("Column stats are successfully loaded.")
    return


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        prog="Quollio Intelligence Agent for Databricks",
        description="Build views and load lineage and stats to Quollio from Databricks using dbt.",
        epilog="Copyright (c) 2024 Quollio Technologies, Inc.",
    )
    parser.add_argument(
        "commands",
        choices=["build_view", "load_lineage", "load_stats"],
        type=str,
        nargs="+",
        help="""
        The command to execute.
        'build_view': Build views using dbt,
        'load_lineage': Load lineage data from created views to Quollio,
        'load_stats': Load stats from created views to Quollio,
        'load_sqllineage': Load lineage data from sql parse result(alpha),
        """,
    )
    parser.add_argument(
        "--host", type=str, action=env_default("DATABRICKS_HOST"), required=False, help="Host for Databricks workspace"
    )
    parser.add_argument(
        "--http_path",
        type=str,
        action=env_default("DATABRICKS_HTTP_PATH"),
        required=False,
        help="HTTP path for a Databricks compute resource (i.e warehouse)",
    )
    parser.add_argument(
        "--port",
        type=int,
        action=env_default("DATABRICKS_PORT"),
        required=False,
        help="Port for Databricks compute resource",
    )
    parser.add_argument(
        "--databricks_client_secret",
        type=str,
        action=env_default("DATABRICKS_CLIENT_SECRET"),
        required=False,
        help="Secret for the service principal",
    )
    parser.add_argument(
        "--databricks_client_id",
        type=str,
        action=env_default("DATABRICKS_CLIENT_ID"),
        required=False,
        help="Client id for the service principal",
    )
    parser.add_argument(
        "--catalog",
        type=str,
        required=False,
        action=env_default("DATABRICKS_TARGET_CATALOG"),
        help="Target database name where the views are built by dbt",
    )
    parser.add_argument(
        "--schema",
        type=str,
        action=env_default("DATABRICKS_TARGET_SCHEMA"),
        required=False,
        help="Target schema name where the views are built by dbt",
    )
    parser.add_argument(
        "--log_level",
        type=str,
        choices=["debug", "info", "warn", "error", "none"],
        action=env_default("LOG_LEVEL"),
        required=False,
        help="The log level for dbt commands. Default value is info",
    )
    parser.add_argument(
        "--api_url",
        type=str,
        action=env_default("QDC_API_URL"),
        required=False,
        help="The base URL of Quollio External API",
    )
    parser.add_argument(
        "--client_id",
        type=str,
        action=env_default("QDC_CLIENT_ID"),
        required=False,
        help="The client id that is created on Quollio console to let clients access Quollio External API",
    )
    parser.add_argument(
        "--client_secret",
        type=str,
        action=env_default("QDC_CLIENT_SECRET"),
        required=False,
        help="The client secrete that is created on Quollio console to let clients access Quollio External API",
    )
    parser.add_argument(
        "--tenant_id",
        type=str,
        action=env_default("TENANT_ID"),
        required=False,
        help="The tenant id (company id) where the lineage and stats are loaded",
    )
    parser.add_argument(
        "--target_tables",
        type=str,
        nargs="*",
        choices=["quollio_lineage_table_level", "quollio_lineage_view_level"],
        action=env_default("DATABRICKS_TARGET_TABLES"),
        required=False,
        help="Target tables you want to create with dbt module. \
              You need to specify this parameter if you want to specify tables, not all ones. \
              Please specify table name with blank delimiter like tableA tableB \
              if you want to create two or more tables",
    )

    args = parser.parse_args()

    conn = db.DatabricksConnectionConfig(
        host=args.host,
        http_path=args.http_path,
        client_id=args.databricks_client_id,
        client_secret=args.databricks_client_secret,
        catalog=args.catalog,
        schema=args.schema,
    )

    if len(args.commands) == 0:
        raise ValueError("No command is provided")

    if "build_view" in args.commands:
        build_view(
            conn=conn,
            target_tables=args.target_tables,
            log_level=args.log_level,
        )

    if "load_lineage" in args.commands:
        qdc_client = qdc.QDCExternalAPIClient(
            base_url=args.api_url, client_id=args.client_id, client_secret=args.client_secret
        )
        load_lineage(conn=conn, qdc_client=qdc_client, tenant_id=args.tenant_id)

    if "load_stats" in args.commands:
        qdc_client = qdc.QDCExternalAPIClient(
            base_url=args.api_url, client_id=args.client_id, client_secret=args.client_secret
        )
        databricks_column_stats(conn=conn, qdc_client=qdc_client, tenant_id=args.tenant_id)



import logging

from quollio_core.profilers.lineage import (
    gen_column_lineage_payload,
    gen_table_lineage_payload,
    parse_snowflake_results,
)
from quollio_core.profilers.sqllineage import SQLLineage
from quollio_core.profilers.stats import gen_table_stats_payload
from quollio_core.repository import qdc, snowflake

logger = logging.getLogger(__name__)


def snowflake_table_to_table_lineage(
    conn: snowflake.SnowflakeConnectionConfig,
    qdc_client: qdc.QDCExternalAPIClient,
    tenant_id: str,
) -> None:
    logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(name)s - %(message)s")
    sf_executor = snowflake.SnowflakeQueryExecutor(conn)
    results = sf_executor.get_query_results(
        query="""
        SELECT
            *
        FROM
           {db}.{schema}.QUOLLIO_LINEAGE_TABLE_LEVEL
        """.format(
            db=conn.account_database,
            schema=conn.account_schema,
        )
    )
    parsed_results = parse_snowflake_results(results=results)
    update_table_lineage_inputs = gen_table_lineage_payload(
        tenant_id=tenant_id,
        endpoint=conn.account_id,
        tables=parsed_results,
    )

    req_count = 0
    for update_table_lineage_input in update_table_lineage_inputs:
        logger.info(
            "Generating table lineage. downstream: {db} -> {schema} -> {table}".format(
                db=update_table_lineage_input.downstream_database_name,
                schema=update_table_lineage_input.downstream_schema_name,
                table=update_table_lineage_input.downstream_table_name,
            )
        )
        status_code = qdc_client.update_lineage_by_id(
            global_id=update_table_lineage_input.downstream_global_id,
            payload=update_table_lineage_input.upstreams.as_dict(),
        )
        if status_code == 200:
            req_count += 1
    logger.info(f"Generating table lineage is finished. {req_count} lineages are ingested.")
    return


def snowflake_column_to_column_lineage(
    conn: snowflake.SnowflakeConnectionConfig,
    qdc_client: qdc.QDCExternalAPIClient,
    tenant_id: str,
) -> None:
    sf_executor = snowflake.SnowflakeQueryExecutor(conn)
    results = sf_executor.get_query_results(
        query="""
        SELECT
            *
        FROM
            {db}.{schema}.QUOLLIO_LINEAGE_COLUMN_LEVEL
        """.format(
            db=conn.account_database,
            schema=conn.account_schema,
        )
    )
    update_column_lineage_inputs = gen_column_lineage_payload(
        tenant_id=tenant_id,
        endpoint=conn.account_id,
        columns=results,
    )

    req_count = 0
    for update_column_lineage_input in update_column_lineage_inputs:
        logger.info(
            "Generating column lineage. downstream: {db} -> {schema} -> {table} -> {column}".format(
                db=update_column_lineage_input.downstream_database_name,
                schema=update_column_lineage_input.downstream_schema_name,
                table=update_column_lineage_input.downstream_table_name,
                column=update_column_lineage_input.downstream_column_name,
            )
        )
        status_code = qdc_client.update_lineage_by_id(
            global_id=update_column_lineage_input.downstream_global_id,
            payload=update_column_lineage_input.upstreams.as_dict(),
        )
        if status_code == 200:
            req_count += 1
    logger.info(f"Generating column lineage is finished. {req_count} lineages are ingested.")
    return


def snowflake_table_level_sqllineage(
    conn: snowflake.SnowflakeConnectionConfig,
    qdc_client: qdc.QDCExternalAPIClient,
    tenant_id: str,
) -> None:
    logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(name)s - %(message)s")
    sf_executor = snowflake.SnowflakeQueryExecutor(conn)
    results = sf_executor.get_query_results(
        query="""
        SELECT
            database_name
            , schema_name
            , query_text
        FROM
           {db}.{schema}.QUOLLIO_SQLLINEAGE_SOURCES
        """.format(
            db=conn.account_database,
            schema=conn.account_schema,
        )
    )
    update_table_lineage_inputs_list = list()
    sql_lineage = SQLLineage()
    for result in results:
        src_tables, dest_table = sql_lineage.get_table_level_lineage_source(
            sql=result["QUERY_TEXT"],
            dialect="snowflake",
            dest_db=result["DATABASE_NAME"],
            dest_schema=result["SCHEMA_NAME"],
        )
        update_table_lineage_inputs = sql_lineage.gen_lineage_input(
            tenant_id=tenant_id, endpoint=conn.account_id, src_tables=src_tables, dest_table=dest_table
        )
        update_table_lineage_inputs_list.append(update_table_lineage_inputs)

    req_count = 0
    for update_table_lineage_input in update_table_lineage_inputs_list:
        logger.info(
            "Generating table lineage. downstream: {db} -> {schema} -> {table}".format(
                db=update_table_lineage_input.downstream_database_name,
                schema=update_table_lineage_input.downstream_schema_name,
                table=update_table_lineage_input.downstream_table_name,
            )
        )
        status_code = qdc_client.update_lineage_by_id(
            global_id=update_table_lineage_input.downstream_global_id,
            payload=update_table_lineage_input.upstreams.as_dict(),
        )
        if status_code == 200:
            req_count += 1
    logger.info(f"Generating table lineage is finished. {req_count} lineages are ingested.")
    return


def _get_target_tables_query(db: str, schema: str) -> str:
    query = """
        SELECT
            DISTINCT
            TABLE_CATALOG
            , TABLE_SCHEMA
            , TABLE_NAME
        FROM
            {db}.{schema}.QUOLLIO_STATS_PROFILING_COLUMNS
        """.format(
        db=db, schema=schema
    )
    return query


def _get_stats_tables_query(db: str, schema: str) -> str:
    query = """
        SELECT
            DISTINCT
            TABLE_CATALOG
            , TABLE_SCHEMA
            , TABLE_NAME
        FROM
            {db}.INFORMATION_SCHEMA.TABLES
        WHERE
            startswith(TABLE_NAME, 'QUOLLIO_STATS_COLUMNS_')
            AND TABLE_SCHEMA = UPPER('{schema}')
        """.format(
        db=db, schema=schema
    )
    return query


def snowflake_table_stats(
    conn: snowflake.SnowflakeConnectionConfig,
    qdc_client: qdc.QDCExternalAPIClient,
    tenant_id: str,
) -> None:
    logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(name)s - %(message)s")
    sf_executor = snowflake.SnowflakeQueryExecutor(conn)

    target_query = _get_target_tables_query(
        db=conn.account_database,
        schema=conn.account_schema,
    )
    target_assets = sf_executor.get_query_results(query=target_query)

    stats_query = _get_stats_tables_query(
        db=conn.account_database,
        schema=conn.account_schema,
    )
    stats_columns = sf_executor.get_query_results(query=stats_query)

    req_count = 0
    for target_asset in target_assets:
        for stats_column in stats_columns:
            stats_query = """
            SELECT
                db_name
                , schema_name
                , table_name
                , column_name
                , max_value
                , min_value
                , null_count
                , cardinality
                , avg_value
                , median_value
                , mode_value
                , stddev_value
            FROM
                {db}.{schema}.{table}
            WHERE
                db_name = '{target_db}'
                and schema_name = '{target_schema}'
                and table_name = '{target_table}'
            """.format(
                db=stats_column["TABLE_CATALOG"],
                schema=stats_column["TABLE_SCHEMA"],
                table=stats_column["TABLE_NAME"],
                target_db=target_asset["TABLE_CATALOG"],
                target_schema=target_asset["TABLE_SCHEMA"],
                target_table=target_asset["TABLE_NAME"],
            )
            stats_result = sf_executor.get_query_results(query=stats_query)
            payloads = gen_table_stats_payload(tenant_id=tenant_id, endpoint=conn.account_id, stats=stats_result)
            for payload in payloads:
                logger.info(
                    "Generating table stats. asset: {db} -> {schema} -> {table} -> {column}".format(
                        db=payload.db,
                        schema=payload.schema,
                        table=payload.table,
                        column=payload.column,
                    )
                )
                status_code = qdc_client.update_stats_by_id(
                    global_id=payload.global_id,
                    payload=payload.body.get_column_stats(),
                )
                if status_code == 200:
                    req_count += 1
    logger.info(f"Generating table stats is finished. {req_count} stats are ingested.")
